#include <Wire.h> 
#include <DHT.h>
#include "RTClib.h"
RTC_DS1307 rtc;
char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
DateTime now;

#include <LiquidCrystal_I2C.h>
// Set the LCD address to 0x3F for a 20 chars and 4 line display
LiquidCrystal_I2C lcd(0x3F, 20, 4);
// Vereinbarungen zur Konfiguration von DHT22 bzw. DHT11
#define DHTPIN 2          // Digitalpin mit dem DHT22
#define DHTTYPE DHT22     // Sensor Typ: DHT 22  (oder für DHT 11: DHT11) 
DHT dht(DHTPIN, DHTTYPE); // DHT-Zugriffsobjekt 

#include <Adafruit_BMP280.h>
Adafruit_BMP280 bmp; // I2C

/////////////////////////////////////////////////////////////////
// DEKLARATIONEN SD Card
/////////////////////////////////////////////////////////////////
#define LOGDATEI "TMP36"
#include <SPI.h>
#include "SdFat.h"

// Chip Selector auf Arduino Board
const uint8_t chipSelect = 53;
// Separatorzeichen für Dateiausgabe
const String SEP = " ; ";
// Zugriff auf Dateisystem der SD Card
SdFat sd;
// Log-Datei
SdFile datei;
// Fehlermeldungen im Flash ablegen.
#define error(msg) sd.errorHalt(F(msg))

String datum, uhrzeit; 
float dhtHumidity;
float dhtTemperature;
float luftdruck;
int   old_day = 0;
int   old_min = 0;

void setup()
{
  Serial.begin(9600); 
  Wire.begin(); 
   if (!bmp.begin()) {  
      Serial.println(F("Could not find a valid BMP280 sensor, check wiring!"));
      while (1);
  }
  initSDCardReader();
  dht.begin(); // DHT22 bzw. DHT11 starten
	lcd.begin(); // initialize the LCD
	// Turn on the blacklight and print a message.
	lcd.backlight();
	lcd.setCursor(0, 0); lcd.print("Temperatur ");
  lcd.setCursor(0, 1); lcd.print("Luftfeuchte");
  lcd.setCursor(0, 2); lcd.print("Luftdruck  ");
  // Initialize DS3231
  Serial.println("Initialize RTC");;
  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }
  //rtc.adjust(DateTime(2018, 1, 21, 20, 30, 0));  //Zeile einmalig aktivieren zum Setzen der aktuellen Zeit und Datum, danach wieder deaktivieren
  if (! rtc.isrunning()) {
    Serial.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    // rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    //  rtc.adjust(DateTime(2018, 1, 21, 20, 29, 0));
  }
  else
  { now = rtc.now();
    old_day = now.day();
  } 
  printTemp();      // Temperatur + Luftfeuchte messen und Ausgabe an Display
  printPressure();  // Luftdruck messen und Ausgabe an Display 
}

void loop()
{   // Datum und Uhrzeit lesen und Ausgabe auf dem Bildschirm und Display
    now = rtc.now();
    // Datum als String zusammensetzen
    datum = "";
    if( now.day() <10) {datum +="0";}; datum +=  now.day(); datum += "."; 
    if( now.month() <10) {datum +="0";}; datum +=  now.month(); datum += "."; 
    datum += now.year(); datum += " ";
     // Uhrzeit als String zusammensetzen
    uhrzeit = "";
    if (now.hour() < 10){ uhrzeit += "0";}; uhrzeit += now.hour(); uhrzeit += ":";
    if (now.minute() < 10){uhrzeit += "0";}; uhrzeit += now.minute(); uhrzeit += ":";
    if (now.second() < 10){uhrzeit += "0";}; uhrzeit += now.second(); 
    lcd.setCursor(0, 3);  // Cursor im Display auf Angang in Zeile 4 setzen
    Serial.print(datum); lcd.print(datum);
    Serial.println(uhrzeit); lcd.print(uhrzeit);
    if (old_min != now.minute())
    {
    // einmal je Minute Messwerte schreiben
    old_min = now.minute(); 
    printTemp();      // Temperatur + Luftfeuchte messen und Ausgabe an Display
    printPressure();  // Luftdruck messen und Ausgabe an Display 
    schreibeMessung(); 
    }
  
    if (old_day != now.day() )
    {  
     old_day = now.day();
     // letzte Datei schliessen
     dateiSchliessen(); 
     // neue Datei öffen
     initSDCardReader();
    }
    delay(1000);
} // end loop

void printPressure() {
    Serial.print("Temperature = ");
    Serial.print(bmp.readTemperature());
    Serial.println(" *C");

    Serial.print("Pressure = ");
    luftdruck = bmp.readPressure() / 100.0;
    Serial.print(luftdruck);
    Serial.println(" mBar");
    lcd.setCursor(12, 2); lcd.print(luftdruck);  
}

void printTemp(){
// Feuchtigkeitswert bestimmen
  dhtHumidity = dht.readHumidity();
  // Temperatur in Celsius auslesen
  dhtTemperature = dht.readTemperature();
  // Sinnvolle Ergebnisse ermittelt?
  if (isnan(dhtHumidity) || isnan(dhtTemperature)) {
    Serial.println("Konnte nicht vom DHT-Sensor lesen.");
    return;
  }
  lcd.setCursor(12, 0); lcd.print(dhtTemperature); lcd.print(" *C");
  lcd.setCursor(12, 1); lcd.print(dhtHumidity); lcd.print(" %"); 
  // Hitzeindex ermitteln  mit Argument "false",  
  // da in Celsius gemessen werden soll, nicht in Fahrenheit
 // hic = dht.computeHeatIndex(dhtTemperature, dhtHumidity, false);
  Serial.print("Feuchtigkeit: "); 
  Serial.print(dhtHumidity); Serial.print(" %\t");
  Serial.println("");
  Serial.print("Temperatur: "); 
  Serial.print(dhtTemperature); Serial.print(" C ");
  Serial.println("");
  //Serial.print("Hitzeindex: "); 
  //Serial.print(hic); Serial.print(" C ");
  Serial.println("");
}

/////////////////////////////////////////////////////////////////
//
// initSDCardReader()
//    Kartenleser initialisieren
//
/////////////////////////////////////////////////////////////////
void initSDCardReader()
{
  const uint8_t NAMENSLAENGE= sizeof(LOGDATEI) - 1;
  char dateiName[13] = LOGDATEI "00.csv"; // Z.B. TMP3604.csv
 
  delay(1000);
  // SD Card mit SPI_HALF_SPEED initialisieren, um Fehler
  // bei Breadboardnutzung zu vermeiden.  Sonst => SPI_FULL_SPEED 
  if (!sd.begin(chipSelect, SPI_HALF_SPEED)) { // Zugriff auf SD?
    sd.initErrorHalt();
  }
 
  // Dateiformat 8.3
  if (NAMENSLAENGE > 6) {
    error("Dateipräfix zu lang");
  }

  // Standarddateiname LOGDATEI  + laufende Nummer, z.B.
  // TMP3603.csv
  // Sobald alle Suffixe 00..09 verbraucht sind,
  // geht es von vorne los: round robin
 
  while (sd.exists(dateiName)) {
    if (dateiName[NAMENSLAENGE + 1] != '9') {
     dateiName[NAMENSLAENGE + 1]++;
    }
    else if (dateiName[NAMENSLAENGE] != '9') {
      dateiName[NAMENSLAENGE + 1] = '0';
      dateiName[NAMENSLAENGE]++;
    }
    else {
      error("Kann Datei nicht erzeugen");
    }
  }
  // Jetzt öffnen:
  if (!datei.open(dateiName, O_CREAT | O_WRITE | O_EXCL)) {
    error("Datei öffnen misslungen!");
  }

  Serial.print(F("Logging auf: "));
  Serial.println(dateiName);

  // Header schreiben
  schreibeHeader();
}

/////////////////////////////////////////////////////////////////
//
// schreibeHeader
//    Schreiben von Header Information in Datei
//
/////////////////////////////////////////////////////////////////
void schreibeHeader() {
  datei.println(F(" Datenlogger mit DHT22 und BME280"));
  datei.print(F("Datum")); 
  datei.print(SEP);
  datei.print(F("Zeit")); 
  datei.print(SEP);
  datei.print(F("Temperatur"));
  datei.print(SEP);
  datei.print(F("Luftfeuchte"));
  datei.print(SEP);
  datei.print(F("Luftdruck"));
  datei.println();
}

/////////////////////////////////////////////////////////////////
//
// schreibeMessung()
//    messwert nehmen und in gewählter Einheit ausgeben
//    und zusammen mit Datum und Uhrzeit in Datei schreiben
//
/////////////////////////////////////////////////////////////////
void schreibeMessung()
{
  datei.print(datum);
  datei.print(SEP);
  datei.print(uhrzeit);
  datei.print(SEP);
  datei.print(dhtTemperature);
  datei.print(SEP);
  datei.print(dhtHumidity);
  datei.print(SEP);
  datei.print(luftdruck);
  datei.println();
   // Dateisync, um Datenverlust zu vermeiden:
  if (!datei.sync() || datei.getWriteError()) {
    error("Schreibfehler!");
  }
}

/////////////////////////////////////////////////////////////////
//
// dateiSchliessen()
//    Datei wieder schließen
//
/////////////////////////////////////////////////////////////////
void dateiSchliessen()
{
  datei.close();
  Serial.println(F("Datei steht bereit!"));
  // SysCall::halt();
}

